@@ -75,8 +75,6 @@ void connection_free(connection_t *cn)
 		list_free(cn->incoming_lines);
 	if (cn->incoming)
 		free(cn->incoming);
-	if (cn->ip_list)
-		list_free(cn->ip_list);
 	if (cn->connecting_data)
 		connecting_data_free(cn->connecting_data);
 	/* conn->user_data */
@@ -232,53 +230,45 @@ static int _write_socket(connection_t *cn, char *message)
 {
 	size_t size;
 	size_t tcount = 0;
-	size_t p_tcount = 0;
 	ssize_t count;
 
 	size = strlen(message);
+	if (size == 0)
+		return WRITE_OK;
+	/* loop if we wrote some data but not everything, or if error is
+	 * EINTR */
 	do {
-		while ((count = write(cn->handle,
-					((const char *)message) + tcount,
-					size - tcount)) > 0) {
+		count = write(cn->handle, ((const char *)message) + tcount,
+					size - tcount);
+		if (count > 0) {
 			tcount += count;
 			if (tcount == size)
-				break;
-			if (tcount - p_tcount == 0) {
-				/* no write at all, we give up */
-				cn->connected = CONN_ERROR;
-				return WRITE_ERROR;
-			}
-			p_tcount = tcount;
+				return WRITE_OK;
 		}
-	} while (count < 0 &&
-		(errno == EAGAIN || errno == EINTR || errno == EINPROGRESS));
+	} while (count > 0 || (count < 0 && errno == EINTR));
 
-#if 0
-	if (count <= 0 && tcount > 0)
-		fatal("shit happens errno:%d count:%d tcount:%d (%s)\n", errno,
-				count, tcount, message);
-#endif
-	if (count <= 0) {
-		/*
-		 * if no fatal error, return WRITE_KEEP, which makes caller
-		 * keep line in its FIFO
-		 *
-		 * Cannot do: we might have written a partial line
-		 * 
-		if (errno == EAGAIN || errno == EINTR || errno == EINPROGRESS)
-			return WRITE_KEEP;
-		 */
+	/* If we reach this point, we have a partial write */
+	assert(count != 0);
 
-		if (cn_is_connected(cn)) {
-			mylog(LOG_DEBUGVERB, "write(fd %d) : %s", cn->handle,
-					strerror(errno));
-			cn->connected = CONN_ERROR;
-		}
-		mylog(LOG_DEBUGVERB, "write : %s", strerror(errno));
-		return WRITE_ERROR;
+	/* if no fatal error, return WRITE_KEEP, which makes caller keep line
+	 * in its FIFO
+	 *
+	 * Shitty: we might have written a partial line, so we hack the line...
+	 * Callers of _write_socket muse provide a writable message
+	 */
+	if (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINPROGRESS) {
+		memmove(message, message + tcount, size - tcount + 1);
+		return WRITE_KEEP;
 	}
-	mylog(LOG_DEBUGVERB, "%d/%d bytes sent !", tcount, size);
-	return WRITE_OK;
+	/* other errors, EPIPE or worse, close the connection, repport error */
+	if (cn_is_connected(cn)) {
+		if (errno != EPIPE)
+			mylog(LOG_INFO, "Broken socket: %s.", strerror(errno));
+		connection_close(cn);
+		cn->connected = CONN_ERROR;
+	}
+	mylog(LOG_DEBUGVERB, "write: %d, %s", cn->handle, strerror(errno));
+	return WRITE_ERROR;
 }
 
 static int write_socket(connection_t *cn, char *line)
@@ -301,7 +291,14 @@ static int real_write_all(connection_t *cn)
 	if (cn == NULL)
 		fatal("real_write_all: wrong arguments");
 
-	while ((line = list_remove_first(cn->outgoing))) {
+	if (cn->partial) {
+		line = cn->partial;
+		cn->partial = NULL;
+	} else {
+		line = list_remove_first(cn->outgoing);
+	}
+
+	do {
 		ret = write_socket(cn, line);
 
 		switch (ret) {
@@ -311,7 +308,8 @@ static int real_write_all(connection_t *cn)
 			return 1;
 		case WRITE_KEEP:
 			/* interrupted or not ready */
-			list_add_first(cn->outgoing, line);
+			assert(cn->partial == NULL);
+			cn->partial = line;
 			return 0;
 		case WRITE_OK:
 			free(line);
@@ -324,37 +322,44 @@ static int real_write_all(connection_t *cn)
 		if (cn->anti_flood)
 			/* one line at a time */
 			break;
-	}
+	} while ((line = list_remove_first(cn->outgoing)));
 	return 0;
 }
 
 void write_line_fast(connection_t *cn, char *line)
 {
 	int r;
-	r = write_socket(cn, line);
-	switch (r) {
-	case WRITE_KEEP:
-		list_add_first(cn->outgoing, bip_strdup(line));
-		break;
-	case WRITE_ERROR:
-		cn->connected = CONN_ERROR;
-		break;
-	case WRITE_OK:
-		break;
-	default:
-		fatal("internal error 7");
-		break;
+	char *nline = bip_strdup(line);
+
+	if (cn->partial) {
+		list_add_first(cn->outgoing, nline);
+	} else {
+		r = write_socket(cn, nline);
+		switch (r) {
+		case WRITE_KEEP:
+			cn->partial = nline;
+			break;
+		case WRITE_ERROR:
+		case WRITE_OK:
+			free(nline);
+			break;
+		default:
+			fatal("internal error 7");
+			break;
+		}
 	}
 }
 
 void write_lines(connection_t *cn, list_t *lines)
 {
 	list_append(cn->outgoing, lines);
+	real_write_all(cn);
 }
 
 void write_line(connection_t *cn, char *line)
 {
 	list_add_last(cn->outgoing, bip_strdup(line));
+	real_write_all(cn);
 }
 
 list_t *read_lines(connection_t *cn, int *error)
@@ -1021,7 +1026,6 @@ static connection_t *connection_init(int anti_flood, int ssl, int timeout,
 	conn->listening = listen;
 	conn->handle = -1;
 	conn->client = 0;
-	conn->ip_list = NULL;
 	conn->connecting_data = NULL;
 #ifdef HAVE_LIBSSL
 	conn->ssl_ctx_h = NULL;
@@ -1278,12 +1282,13 @@ static int bip_ssl_verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 
 	/* in basic mode (mode 1), accept a leaf certificate if we can find it
 	 * in the store */
-	if (c->ssl_check_mode == SSL_CHECK_BASIC && depth == 0 && result == 0 &&
+	if (c->ssl_check_mode == SSL_CHECK_BASIC && result == 0 &&
 			(err == X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY ||
 			 err == X509_V_ERR_CERT_UNTRUSTED ||
 			 err == X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE ||
 			 err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT ||
-			 err == X509_V_ERR_CERT_HAS_EXPIRED)) {
+			 err == X509_V_ERR_CERT_HAS_EXPIRED ||
+			 err == X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN)) {
 
 		if (X509_STORE_get_by_subject(ctx, X509_LU_X509,
 				X509_get_subject_name(err_cert), &xobj) > 0 &&
@@ -1440,7 +1445,7 @@ static connection_t *_connection_new_SSL(char *dsthostname, char *dstport,
 	case SSL_CHECK_BASIC:
 		SSL_CTX_set_verify(conn->ssl_ctx_h, SSL_VERIFY_PEER,
 				bip_ssl_verify_callback);
-		SSL_CTX_set_verify_depth(conn->ssl_ctx_h, 0);
+		/* SSL_CTX_set_verify_depth(conn->ssl_ctx_h, 0); */
 		break;
 	case SSL_CHECK_CA:
 		SSL_CTX_set_verify(conn->ssl_ctx_h, SSL_VERIFY_PEER,
