@@ -126,18 +126,18 @@ list_t *channel_name_list(struct channel *c)
 			len = 0;
 		}
 		if (len != 0) {
-			strncat(str, " ", NAMESIZE);
+			strcat(str, " ");
 			len++;
 		}
 		if (ovmask & NICKOP)
-			strncat(str, "@", NAMESIZE);
+			strcat(str, "@");
 		else if (ovmask & NICKHALFOP)
-			strncat(str, "%", NAMESIZE);
+			strcat(str, "%");
 		else if (ovmask & NICKVOICED)
-			strncat(str, "+", NAMESIZE);
+			strcat(str, "+");
 		len++;
 
-		strncat(str, nick, NAMESIZE);
+		strcat(str, nick);
 		len += strlen(nick);
 		assert(len < NAMESIZE);
 	}
@@ -182,10 +182,10 @@ void irc_compute_lag(struct link_server *is)
 int irc_lags_out(struct link_server *is)
 {
 	if (is->lag > LAGOUT_TIME) {
-		mylog(LOG_ERROR, "%s Lags out! closing", LINK(is)->name);
+		mylog(LOG_ERROR, "[%s] Lags out! closing", LINK(is)->name);
 		return 1;
 	} else {
-		mylog(LOG_DEBUG, "%s lag : %d\n", LINK(is)->name, is->lag);
+		mylog(LOG_DEBUG, "[%s] lag : %d\n", LINK(is)->name, is->lag);
 		return 0;
 	}
 }
@@ -216,7 +216,7 @@ static void irc_server_connected(struct link_server *server)
         LINK(server)->s_state = IRCS_CONNECTED;
         LINK(server)->s_conn_attempt = 0;
 
-	mylog(LOG_INFO, "Connected to server %s for user %s",
+	mylog(LOG_INFO, "[%s] Connected for user %s",
 			LINK(server)->name, LINK(server)->user->name);
 
         irc_server_join(server);
@@ -481,9 +481,12 @@ int irc_dispatch_server(bip_t *bip, struct link_server *server,
 	if (ret == OK_COPY) {
 		int i;
 		for (i = 0; i < LINK(server)->l_clientc; i++) {
-			char *s = irc_line_to_string(line);
-			write_line(CONN(LINK(server)->l_clientv[i]), s);
-			free(s);
+			if (TYPE(LINK(server)->l_clientv[i]) ==
+					IRC_TYPE_CLIENT) {
+				char *s = irc_line_to_string(line);
+				write_line(CONN(LINK(server)->l_clientv[i]), s);
+				free(s);
+			}
 		}
 	}
 	if (ret == OK_COPY_WHO && LINK(server)->who_client) {
@@ -687,7 +690,8 @@ void irc_cli_backlog(struct link_client *ic, int hours)
 				LINK(ic)->l_server->nick, hours);
 		if (bllines) {
 			if (!list_is_empty(bllines)) {
-				mylog(LOG_INFO, "backlogging: %s", bl);
+				mylog(LOG_INFO, "[%s] backlogging: %s",
+						LINK(ic)->name, bl);
 				write_lines(CONN(ic), bllines);
 			}
 			list_free(bllines);
@@ -736,8 +740,8 @@ static int irc_cli_startup(bip_t *bip, struct link_client *ic,
 	}
 
 	if (!LINK(ic))
-		mylog(LOG_ERROR, "Invalid credentials (user:%s connection:%s)",
-				user, connname);
+		mylog(LOG_ERROR, "[%s] Invalid credentials (user: %s)",
+				 connname, user);
 	free(user);
 	free(connname);
 	free(pass);
@@ -1058,12 +1062,16 @@ static int irc_cli_join(struct link_client *irc, struct line *line)
 
 static int irc_cli_part(struct link_client *irc, struct line *line)
 {
+	struct chan_info *ci;
+	char *cname;
+
 	if (irc_line_count(line) != 2 && irc_line_count(line) != 3)
 		return ERR_PROTOCOL;
 
-	struct chan_info *ci;
+	cname = (char *)irc_line_elem(line, 1);
+
 	if ((ci = hash_remove_if_exists(&LINK(irc)->chan_infos,
-					irc_line_elem(line, 1))) != NULL) {
+					cname)) != NULL) {
 		list_remove(&LINK(irc)->chan_infos_order, ci);
 		free(ci->name);
 		if (ci->key)
@@ -1505,6 +1513,8 @@ static int irc_part(struct link_server *server, struct line *line)
 		log_part(LINK(server)->log, line->origin, s_chan,
 			irc_line_count(line) == 3 ? irc_line_elem(line, 2) :
 				NULL);
+		log_reset_store(LINK(server)->log, s_chan);
+		log_drop(LINK(server)->log, s_chan);
 
 		hash_remove(&server->channels, s_chan);
 		channel_free(channel);
@@ -1781,6 +1791,8 @@ static int irc_kick(struct link_server *server, struct line *line)
 				irc_line_elem(line, 2),
 				irc_line_count(line) == 4 ?
 					irc_line_elem(line, 3) : NULL);
+		log_reset_store(LINK(server)->log, channel->name);
+		log_drop(LINK(server)->log, channel->name);
 
 		if (LINK(server)->autojoin_on_kick) {
 			if (!channel->key)
@@ -2062,7 +2074,7 @@ static void server_setup_reconnect_timer(struct link *link)
 		if (timer > RECONN_TIMER_MAX)
 			timer = RECONN_TIMER_MAX;
 	}
-	mylog(LOG_ERROR, "%s dead, reconnecting in %d seconds", link->name,
+	mylog(LOG_ERROR, "[%s] reconnecting in %d seconds", link->name,
 			timer);
 	link->recon_timer = timer;
 }
@@ -2147,8 +2159,8 @@ connection_t *irc_server_connect(struct link *link)
 
 	link->s_conn_attempt++;
 
-	mylog(LOG_INFO, "Connecting user '%s' to network '%s' using server "
-		"%s:%d", link->user->name, link->name,
+	mylog(LOG_INFO, "[%s] Connecting user '%s' using server "
+		"%s:%d", link->name, link->user->name,
 		link->network->serverv[link->cur_server].host,
 		link->network->serverv[link->cur_server].port);
 	conn = connection_new(link->network->serverv[link->cur_server].host,
@@ -2164,7 +2176,7 @@ connection_t *irc_server_connect(struct link *link)
 				CONNECT_TIMEOUT);
 	assert(conn);
 	if (conn->handle == -1) {
-		mylog(LOG_INFO, "Cannot connect.");
+		mylog(LOG_INFO, "[%s] Cannot connect.", link->name);
 		connection_free(conn);
 		server_next(link);
 		return NULL;
@@ -2432,11 +2444,11 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 	list_t *linel = read_lines(conn, &err);
 	if (err) {
 		if (TYPE(lc) == IRC_TYPE_SERVER) {
-			mylog(LOG_ERROR, "read_lines error, closing %s ...",
+			mylog(LOG_ERROR, "[%s] read_lines error, closing...",
 					LINK(lc)->name);
 			irc_server_shutdown(LINK(lc)->l_server);
 		} else {
-			mylog(LOG_ERROR, "read_lines error, closing...");
+			mylog(LOG_ERROR, "client read_lines error, closing...");
 		}
 		goto prot_err;
 	}
@@ -2454,7 +2466,8 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 
 		line = irc_line_new_from_string(line_s);
 		if (!line) {
-			mylog(LOG_ERROR, "Error in protocol, closing...");
+			mylog(LOG_ERROR, "[%s] Error in protocol, closing...",
+					LINK(lc)->name);
 			free(line_s);
 			goto prot_err_lines;
 		}
@@ -2463,8 +2476,8 @@ void bip_on_event(bip_t *bip, connection_t *conn)
 		irc_line_free(line);
 		free(line_s);
 		if (r == ERR_PROTOCOL) {
-			mylog(LOG_ERROR, "Error in protocol, "
-					"closing...");
+			mylog(LOG_ERROR, "[%s] Error in protocol, closing...",
+					LINK(lc)->name);
 			goto prot_err_lines;
 		}
 		if (r == ERR_AUTH)
