@@ -15,7 +15,7 @@
 #define IRC_H
 #include "connection.h"
 #include "line.h"
-
+#include "log.h"
 
 #define ERR_PROTOCOL (-1)
 #define ERR_AUTH (-2)
@@ -25,24 +25,20 @@
 #define OK_COPY_CLI (4)
 #define OK_COPY_WHO (5)
 
-#define P_SERV "bip.bip.bip"
+#define P_SERV "b.i.p"
 #define S_PING "BIPPING"
-#define P_IRCMASK "-bip!bip@bip.bip.bip"
+#define P_IRCMASK "-bip!bip@" P_SERV
 
 struct server {
 	char *host;
 	unsigned short port;
 };
 
-#define server_new() calloc(sizeof(struct server), 1)
+#define server_new() bip_calloc(sizeof(struct server), 1)
 
 #define NICKOP 1
-#define NICKVOICED 2
-
-struct nick {
-	char *name;
-	int ovmask;
-};
+#define NICKHALFOP (1<<1)
+#define NICKVOICED (1<<2)
 
 struct channel {
 	char *name;
@@ -53,7 +49,7 @@ struct channel {
 	char type;
 	char *creator;
 	char *create_ts;
-	hash_t nicks;
+	hash_t ovmasks;
 	int running_names;
 };
 
@@ -78,12 +74,13 @@ struct user {
 	char *default_realname;
 
 	/* backlog options */
-	int backlog;
+	int backlog:1;
 	int backlog_lines;
-	int always_backlog;
-	int bl_msg_only;
-	int backlog_no_timestamp;
-	int blreset_on_talk;
+	int always_backlog:1;
+	int bl_msg_only:1;
+	int backlog_no_timestamp:1;
+	int blreset_on_talk:1;
+	int blreset_connection:1;
 
 #ifdef HAVE_LIBSSL
 	int ssl_check_mode;
@@ -92,7 +89,7 @@ struct user {
 #endif
 
 	hash_t connections;
-	int in_use; /* for mark and sweep on reload */
+	int in_use:1; /* for mark and sweep on reload */
 };
 
 struct network
@@ -132,8 +129,10 @@ struct link {
 
 	/** link options */
 
-	int follow_nick;
-	int ignore_first_nick;
+	int follow_nick:1;
+	int ignore_first_nick:1;
+	int autojoin_on_kick:1;
+	int ignore_server_capab:1;
 	list_t on_connect_send;
 	char *no_client_away_msg;
 	char *away_nick;
@@ -215,7 +214,7 @@ struct chan_info {
 	int backlog;
 };
 
-#define chan_info_new() calloc(sizeof(struct chan_info), 1)
+#define chan_info_new() bip_calloc(sizeof(struct chan_info), 1)
 
 struct link_server {
 	struct link_connection _link_c;
@@ -265,5 +264,9 @@ void irc_client_free(struct link_client *);
 struct link *irc_link_new();
 void link_kill(bip_t *bip, struct link *);
 void unbind_from_link(struct link_client *ic);
+char *nick_from_ircmask(const char *mask);
+void irc_cli_backlog(struct link_client *ic, int hours);
+#define BIP_FAKEMASK "!bip@bip.bip.bip"
+
 #endif
 
