@@ -30,7 +30,8 @@ static int _log_write(log_t *logdata, logstore_t *lf, const char *d,
 		const char *str);
 static char *_log_wrap(const char *dest, const char *line);
 void logfile_free(logfile_t *lf);
-static void log_drop(log_t *log, const char *storename);
+void log_drop(log_t *log, const char *storename);
+static void log_reset(logstore_t *store);
 
 #define BOLD_CHAR 0x02
 #define LAMESTRING "!bip@" P_SERV " PRIVMSG "
@@ -132,12 +133,12 @@ void replace_var(char *str, char *var, char *value, unsigned int max)
 
 char *log_build_filename(log_t *logdata, const char *destination)
 {
-	char *logfile, year[5], day[3], month[3], *tmp, *logdir;
+	char *logfile, year[5], day[3], month[3], hour[3], *tmp, *logdir;
 	struct tm *now;
 	time_t s;
 	char *dest = bip_strdup(destination);
-	strtolower(dest);
 
+	strtolower(dest);
 	logfile = (char *)bip_malloc(MAX_PATH_LEN + 1);
 
 	time(&s);
@@ -145,6 +146,7 @@ char *log_build_filename(log_t *logdata, const char *destination)
 	snprintf(year, 5, "%04d", now->tm_year + 1900);
 	snprintf(day, 3, "%02d", now->tm_mday);
 	snprintf(month, 3, "%02d", now->tm_mon + 1);
+	snprintf(hour, 3, "%02d", now->tm_hour);
 	snprintf(logfile, MAX_PATH_LEN, "%s/%s", conf_log_root,
 			conf_log_format);
 	replace_var(logfile, "%u", logdata->user->name, MAX_PATH_LEN);
@@ -153,6 +155,7 @@ char *log_build_filename(log_t *logdata, const char *destination)
 	replace_var(logfile, "%Y", year, MAX_PATH_LEN);
 	replace_var(logfile, "%d", day, MAX_PATH_LEN);
 	replace_var(logfile, "%m", month, MAX_PATH_LEN);
+	replace_var(logfile, "%h", hour, MAX_PATH_LEN);
 
 	logdir = bip_strdup(logfile);
 
@@ -179,7 +182,7 @@ void log_updatelast(logfile_t *lf)
 	localtime_r(&t, &lf->last_log);
 }
 
-void log_reset(logstore_t *store)
+static void log_reset(logstore_t *store)
 {
 	logfile_t *olf;
 
@@ -256,18 +259,23 @@ static int log_add_file(log_t *logdata, const char *destination,
 	FILE *f;
 	logstore_t *store;
 	char *uniq_fname;
+	char *canonical_fname = NULL;
 	logfile_t *lf = NULL;
 
 	if (conf_log) {
-		if (log_has_file(logdata, filename))
+		if (log_has_file(logdata, filename)) {
+			canonical_fname = bip_strdup(filename);
 			uniq_fname = filename_uniq(filename);
-		else
+		} else {
+			canonical_fname = bip_strdup(filename);
 			uniq_fname = bip_strdup(filename);
+		}
 		f = fopen(uniq_fname, "a+");
 		if (!f) {
 			mylog(LOG_ERROR, "fopen(%s) %s", uniq_fname,
 					strerror(errno));
 			free(uniq_fname);
+			free(canonical_fname);
 			return 0;
 		}
 
@@ -275,6 +283,7 @@ static int log_add_file(log_t *logdata, const char *destination,
 			mylog(LOG_ERROR, "fseek(%s) %s", uniq_fname,
 					strerror(errno));
 			free(uniq_fname);
+			free(canonical_fname);
 			fclose(f);
 			return 0;
 		}
@@ -283,16 +292,18 @@ static int log_add_file(log_t *logdata, const char *destination,
 		lf->file = f;
 		lf->len = ftell(f);
 		lf->filename = uniq_fname;
+		lf->canonical_filename = canonical_fname;
 		log_updatelast(lf);
 	}
 
 	store = hash_get(&logdata->logfgs, destination);
-
 	if (!store) {
 		store = bip_calloc(sizeof(logstore_t), 1);
 		list_init(&store->file_group, NULL);
 		store->name = bip_strdup(destination);
 		store->skip_advance = 0;
+		if (lf)
+			store->file_offset = lf->len;
 		hash_insert(&logdata->logfgs, destination, store);
 	}
 
@@ -305,7 +316,6 @@ static int log_add_file(log_t *logdata, const char *destination,
 		list_add_last(&store->file_group, lf);
 		if (list_it_item(&store->file_it) == NULL)
 			list_it_init_last(&store->file_group, &store->file_it);
-		store->file_offset = lf->len;
 	}
 	return 1;
 }
@@ -321,6 +331,8 @@ void logfile_free(logfile_t *lf)
 		fclose(lf->file);
 	if (lf->filename)
 		free(lf->filename);
+	if (lf->canonical_filename)
+		free(lf->canonical_filename);
 	free(lf);
 }
 
@@ -329,8 +341,6 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 	logfile_t *lf;
 	logstore_t *store;
 	char *filename = NULL;
-	time_t t;
-	struct tm *ltime;
 	struct link *l;
 
 	store = hash_get(&logdata->logfgs, destination);
@@ -372,10 +382,14 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 	}
 
 	/* This is reached if store already exists */
+	lf = list_get_last(&store->file_group);
+
+	time_t t;
+	struct tm *ltime;
+
 	time(&t);
 	ltime = localtime(&t);
-	lf = list_get_last(&store->file_group);
-	if (ltime->tm_mday != lf->last_log.tm_mday) {
+	if (ltime->tm_hour != lf->last_log.tm_hour) {
 		logfile_t *oldlf;
 
 		/* day changed, we might want to rotate logfile */
@@ -383,14 +397,16 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 		if (!filename)
 			return NULL;
 
-		if (strcmp(lf->filename, filename) == 0) {
+		if (strcmp(lf->canonical_filename, filename) == 0) {
 			/* finally we don't */
 			free(filename);
 			return store;
 		}
 
-		/* we do want do rotate logfiles */
-		mylog(LOG_DEBUG, "Rotating logfile for %s from", destination);
+		/* we do want to rotate logfiles */
+		mylog(LOG_DEBUG, "Rotating logfile for %s %s %s", destination,
+				lf->filename, filename);
+
 		oldlf = list_get_last(&store->file_group);
 		if (!log_add_file(logdata, destination, filename)) {
 			free(filename);
@@ -613,15 +629,12 @@ void log_mode(log_t *logdata, const char *ircmask, const char *channel,
 
 void log_disconnected(log_t *logdata)
 {
-	logstore_t *store;
 	hash_iterator_t hi;
 	snprintf(logdata->buffer, LOGLINE_MAXLEN, "%s -!- Disconnected"
 			" from server...", timestamp());
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
-			hash_it_next(&hi)) {
-		store = hash_it_item(&hi);
+			hash_it_next(&hi))
 		log_write(logdata, hash_it_key(&hi), logdata->buffer);
-	}
 }
 
 void log_ping_timeout(log_t *logdata)
@@ -664,7 +677,7 @@ void log_store_free(logstore_t *store)
 	logfile_t *lf;
 
 	log_reset(store);
-	if ((lf = list_remove_first(&store->file_group)))
+	while ((lf = list_remove_first(&store->file_group)))
 		logfile_free(lf);
 	free(store->name);
 	if (store->memlog)
@@ -672,7 +685,7 @@ void log_store_free(logstore_t *store)
 	free(store);
 }
 
-static void log_drop(log_t *log, const char *storename)
+void log_drop(log_t *log, const char *storename)
 {
 	logstore_t *store;
 
@@ -684,11 +697,23 @@ void log_reset_all(log_t *logdata)
 {
 	logstore_t *store;
 	hash_iterator_t hi;
+	list_t drop;
+
+	list_init(&drop, NULL);
 
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
 			hash_it_next(&hi)) {
 		store = hash_it_item(&hi);
-		log_reset(store);
+		if (ischannel(*hash_it_key(&hi)))
+			log_reset(store);
+		else
+			list_add_last(&drop, strdup(hash_it_key(&hi)));
+	}
+
+	char *name;
+	while ((name = list_remove_first(&drop))) {
+		log_drop(logdata, name);
+		free(name);
 	}
 }
 
@@ -697,8 +722,11 @@ void log_reset_store(log_t *log, const char *storename)
 	logstore_t *store;
 
 	store = hash_get(&log->logfgs, storename);
-	if (store)
+	if (store) {
 		log_reset(store);
+		if (!ischannel(*storename))
+			log_drop(log, storename);
+	}
 }
 
 void log_client_none_connected(log_t *logdata)
@@ -721,6 +749,7 @@ void log_advance_backlogs(log_t* ld, logstore_t *store)
 {
 	int c;
 
+	//mylog(LOG_ERROR, "LOG: log_advance_backlogs %s", store->name);
 	if (!store->track_backlog)
 		return;
 
@@ -734,6 +763,7 @@ void log_advance_backlogs(log_t* ld, logstore_t *store)
 
 	logfile_t *lf;
 	while ((lf = list_it_item(&store->file_it))) {
+		//mylog(LOG_ERROR, "LOG: %s %d", lf->filename, store->file_offset);
 		if (!lf->file) {
 			lf->file = fopen(lf->filename, "r");
 			if (!lf->file) {
@@ -744,17 +774,24 @@ void log_advance_backlogs(log_t* ld, logstore_t *store)
 			}
 		}
 		if (fseek(lf->file, store->file_offset, SEEK_SET)) {
+			//mylog(LOG_ERROR, "LOG: fseek FAIL %s %d", lf->filename, store->file_offset);
 			log_reinit(store);
 			return;
 		}
 
 		while ((c = fgetc(lf->file)) != EOF) {
 			store->file_offset++;
-			if (c == '\n')
+			if (c == '\n') {
+				//mylog(LOG_ERROR, "LOG: %s advanced at %d", lf->filename, store->file_offset);
 				return;
+			}
 		}
-		if (lf == list_get_last(&store->file_group))
+		//mylog(LOG_ERROR, "LOG: %s advanced at %d", lf->filename, store->file_offset);
+		if (lf == list_get_last(&store->file_group)) {
+			//mylog(LOG_ERROR, "LOG: %s is last of file_group", lf->filename, store->file_offset);
 			return;
+		}
+		//mylog(LOG_ERROR, "LOG: next file %s %d", lf->filename, store->file_offset);
 		fclose(lf->file);
 		lf->file = NULL;
 		list_it_next(&store->file_it);
@@ -939,6 +976,8 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 	char *buf, *logbr;
 	int close = 0;
 
+	//mylog(LOG_ERROR, "log_backread_file store:", store->name);
+
 	if (!lf->file) {
 		lf->file = fopen(lf->filename, "r");
 		if (!lf->file) {
@@ -953,8 +992,10 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 	}
 
 	if (!start && list_it_item(&store->file_it) == lf) {
-		mylog(LOG_DEBUG, "Seeking %s to %d", lf->filename,
+		/*
+		mylog(LOG_ERROR, "bread Seeking %s to %d", lf->filename,
 				store->file_offset);
+		*/
 		if (fseek(lf->file, store->file_offset, SEEK_SET)) {
 			mylog(LOG_ERROR, "Can't seek in %s", lf->filename);
 			list_add_last(res, _log_wrap(store->name,
@@ -962,7 +1003,7 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 			return 0;
 		}
 	} else {
-		mylog(LOG_DEBUG, "Seeking %s to %d", lf->filename, 0);
+		//mylog(LOG_ERROR, "bread Seeking %s to %d", lf->filename, 0);
 		if (fseek(lf->file, 0, SEEK_SET)) {
 			mylog(LOG_ERROR, "Can't seek in %s", lf->filename);
 			list_add_last(res, _log_wrap(store->name,
@@ -1012,6 +1053,7 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 		lf->file = NULL;
 	}
 	free(buf);
+	//mylog(LOG_ERROR, "end of log_backread_file store: %s", store->name);
 	return 1;
 }
 
@@ -1302,7 +1344,7 @@ list_t *backlog_lines(log_t *log, const char *bl, const char *cli_nick,
 	else
 		dest = cli_nick;
 
-	if (log_has_backlog(log, bl)) {
+	if (log_has_backlog(log, bl) || hours) {
 		if (hours == 0)
 			ret = log_backread(log, bl, dest);
 		else
