@@ -18,6 +18,8 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
+#include <stdint.h>
+#include <limits.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
@@ -27,6 +29,57 @@
 extern int conf_log_level;
 extern int conf_log_system;
 extern int errno;
+extern FILE *conf_global_log_file;
+
+void memory_fatal(void)
+{
+	fflush(conf_global_log_file);
+#define OOMMSG "Out of memory.\n"
+	fwrite(OOMMSG, 1, strlen(OOMMSG), conf_global_log_file);
+#undef OOMMSG
+	fflush(conf_global_log_file);
+	exit(28);
+}
+
+void *bip_malloc(size_t size)
+{
+	void *r;
+
+	assert(size < INT_MAX / 4);
+
+	r = malloc(size);
+	if (!r)
+		memory_fatal();
+	return r;
+}
+
+void *bip_calloc(size_t nmemb, size_t size)
+{
+	void *r = calloc(nmemb, size);
+	if (!r)
+		memory_fatal();
+	return r;
+}
+
+void *bip_realloc(void *ptr, size_t size)
+{
+	void *r;
+
+	assert(size < INT_MAX / 4);
+
+	r = realloc(ptr, size);
+	if (size > 0 && r == NULL)
+		memory_fatal();
+	return r;
+}
+
+char *bip_strdup(const char *str)
+{
+	char *r = strdup(str);
+	if (!r)
+		memory_fatal();
+	return r;
+}
 
 /*
  * <nick> ::= <letter> { <letter> | <number> | <special> }
@@ -104,8 +157,6 @@ char *checkmode2text(int v)
 }
 #endif
 
-extern FILE *conf_global_log_file;
-
 void _mylog(int level, char *fmt, va_list ap)
 {
 	char *prefix;
@@ -157,15 +208,33 @@ void mylog(int level, char *fmt, ...)
 	va_end(ap);
 }
 
+#ifdef HAVE_BACKTRACE
+#include <execinfo.h>
+
+void print_trace(void)
+{
+	void *array[32];
+	size_t size;
+
+	size = backtrace(array, 32);
+	fflush(conf_global_log_file);
+	backtrace_symbols_fd(array, size, fileno(conf_global_log_file));
+}
+#endif
+
 extern char *conf_pid_file;
 void fatal(char *fmt, ...)
 {
 	va_list ap;
+
 	va_start(ap, fmt);
-
 	_mylog(LOG_FATAL, fmt, ap);
-
 	va_end(ap);
+
+#ifdef HAVE_BACKTRACE
+	print_trace();
+#endif
+
 	exit(200);
 }
 
@@ -173,32 +242,24 @@ void fatal(char *fmt, ...)
  * list handling functions
  */
 
-struct list_item {
-	struct list_item *next;
-	struct list_item *prev;
-	void *ptr;
-};
-
-int list_ptr_cmp(void *a, void *b)
+int list_ptr_cmp(const void *a, const void *b)
 {
 	if (a == b)
 		return 0;
 	return -1;
 }
 
-void list_init(list_t *l, int (*cmp)(void *, void *))
+void list_init(list_t *l, int (*cmp)(const void *, const void *))
 {
 	l->first = NULL;
 	l->last = NULL;
 	l->cmp = cmp;
 }
 
-list_t *list_new(int (*cmp)(void *, void *))
+list_t *list_new(int (*cmp)(const void *, const void *))
 {
 	list_t *l;
-	l = malloc(sizeof(list_t));
-	if (!l)
-		fatal("malloc");
+	l = bip_malloc(sizeof(list_t));
 	list_init(l, cmp);
 	return l;
 }
@@ -206,9 +267,7 @@ list_t *list_new(int (*cmp)(void *, void *))
 static struct list_item *list_item(void *ptr)
 {
 	struct list_item *l;
-	l = malloc(sizeof(struct list_item));
-	if (!l)
-		fatal("malloc");
+	l = bip_malloc(sizeof(struct list_item));
 	l->ptr = ptr;
 	l->next = NULL;
 	l->prev = NULL;
@@ -219,6 +278,7 @@ void list_add_first(list_t *list, void *ptr)
 {
 	struct list_item *li;
 
+	assert(list);
 	if (!ptr)
 		fatal("Cannot add NULL ptr to list.");
 	li = list_item(ptr);
@@ -233,8 +293,8 @@ void list_add_first(list_t *list, void *ptr)
 
 void list_add_first_uniq(list_t *list, void *ptr)
 {
-	if (!ptr)
-		fatal("Cannot add NULL ptr to list.");
+	assert(list);
+	assert_msg(ptr, "Cannot add NULL ptr to list.");
 	if (list_get(list, ptr))
 		return;
 	list_add_first(list, ptr);
@@ -244,8 +304,8 @@ void list_add_last(list_t *list, void *ptr)
 {
 	struct list_item *li;
 
-	if (!ptr)
-		fatal("Cannot add NULL ptr to list.");
+	assert(list);
+	assert_msg(ptr, "Cannot add NULL ptr to list.");
 	li = list_item(ptr);
 	if (!list->first) {
 		list->first = list->last = li;
@@ -258,6 +318,7 @@ void list_add_last(list_t *list, void *ptr)
 
 void *list_get_first(list_t *list)
 {
+	assert(list);
 	if (!list->first)
 		return NULL;
 	return list->first->ptr;
@@ -265,6 +326,7 @@ void *list_get_first(list_t *list)
 
 void *list_get_last(list_t *list)
 {
+	assert(list);
 	if (!list->last)
 		return NULL;
 	return list->last->ptr;
@@ -273,8 +335,10 @@ void *list_get_last(list_t *list)
 void *list_remove_first(list_t *list)
 {
 	struct list_item *l;
-	void *ptr = list_get_first(list);
+	void *ptr;
 
+	assert(list);
+	ptr = list_get_first(list);
 	if (!ptr)
 		return NULL;
 	l = list->first;
@@ -288,7 +352,10 @@ void *list_remove_first(list_t *list)
 void *list_remove_last(list_t *list)
 {
 	struct list_item *l;
-	void *ptr = list_get_last(list);
+	void *ptr;
+
+	assert(list);
+	ptr = list_get_last(list);
 
 	if (!ptr)
 		return NULL;
@@ -300,14 +367,14 @@ void *list_remove_last(list_t *list)
 	return ptr;
 }
 
-void *list_remove_if_exists(list_t *list, void *ptr)
+void *list_remove_if_exists(list_t *list, const void *ptr)
 {
 	list_iterator_t li;
 	int debug = 0;
 	void *ret = 0;
 
-	if (!list->cmp)
-		fatal("list_remove: list does not have a cmp function\n");
+	assert(list);
+	assert_msg(list->cmp, "list does not have a cmp function");
 
 	for (list_it_init(list, &li); list_it_item(&li); list_it_next(&li)) {
 		if (list->cmp(list_it_item(&li), ptr) == 0) {
@@ -322,7 +389,7 @@ void *list_remove_if_exists(list_t *list, void *ptr)
 	return NULL;
 }
 
-void *list_remove(list_t *list, void *ptr)
+void *list_remove(list_t *list, const void *ptr)
 {
 	void *ret;
 	if (!(ret = list_remove_if_exists(list, ptr)))
@@ -330,12 +397,11 @@ void *list_remove(list_t *list, void *ptr)
 	return ret;
 }
 
-void *list_get(list_t *list, void *ptr)
+void *list_get(list_t *list, const void *ptr)
 {
 	struct list_item *it;
 
-	if (!list->cmp)
-		fatal("list_get: list does not have a cmp function\n");
+	assert_msg(list->cmp, "list_get: list does not have a cmp function");
 
 	for (it = list->first; it; it = it->next) {
 		if (list->cmp(it->ptr, ptr) == 0)
@@ -346,37 +412,29 @@ void *list_get(list_t *list, void *ptr)
 
 int list_is_empty(list_t *l)
 {
+	assert(l);
 	return (l->first ? 0 : 1);
 }
 
 void list_it_init(list_t *list, list_iterator_t *ti)
 {
+	assert(list && ti);
 	ti->list = list;
 	ti->cur = list->first;
 	ti->next = NULL;
 }
 
-void list_it_next(list_iterator_t *ti)
+void list_it_init_last(list_t *list, list_iterator_t *ti)
 {
-	if (ti->cur) {
-		if (ti->next)
-			fatal("list_it_next: inconsistent interator state");
-		ti->cur = ti->cur->next;
-	} else if (ti->next) {
-		ti->cur = ti->next;
-		ti->next = NULL;
-	}
-}
-
-void *list_it_item(list_iterator_t *ti)
-{
-	if (!ti->cur)
-		return NULL;
-	return ti->cur->ptr;
+	assert(list && ti);
+	ti->list = list;
+	ti->cur = list->last;
+	ti->next = NULL;
 }
 
 void *list_it_remove(list_iterator_t *li)
 {
+	assert(li);
 	if (!li->cur)
 		return NULL;
 
@@ -400,17 +458,26 @@ void *list_it_remove(list_iterator_t *li)
 
 void list_free(list_t *t)
 {
-	if (t->first != NULL)
-		fprintf(stderr, "Warning, freeing non empty list\n");
+	assert(t);
+	assert(list_is_empty(t));
 	free(t);
 }
 
-void list_append(list_t *src, list_t *dest)
+void list_append(list_t *dest, list_t *src)
 {
-	list_iterator_t it;
-
-	for (list_it_init(src, &it); list_it_item(&it); list_it_next(&it))
-		list_add_last(dest, list_it_item(&it));
+	assert(dest && src);
+	if (src->last == NULL)
+		return;
+	if (dest->first == NULL) {
+		dest->first = src->first;
+		dest->last = src->last;
+		src->first = src->last = NULL;
+		return;
+	}
+	src->first->prev = dest->last;
+	dest->last->next = src->first;
+	dest->last = src->last;
+	src->first = src->last = NULL;
 }
 
 /*
@@ -422,7 +489,7 @@ struct hash_item {
 	void *item;
 };
 
-static int hash_item_nocase_cmp(struct hash_item *a, char *b)
+static int hash_item_nocase_cmp(const struct hash_item *a, const char *b)
 {
 	return strcasecmp(a->key, b);
 }
@@ -436,16 +503,19 @@ void hash_init(hash_t *h, int options)
 {
 	int i;
 
+	assert(h);
 	memset(h, 0, sizeof(hash_t));
 	for (i = 0; i < 256; i++) {
 		switch (options) {
 		case HASH_NOCASE:
 			list_init(&h->lists[i],
-				(int (*)(void*,void*))hash_item_nocase_cmp);
+				(int (*)(const void*, const void*))
+					hash_item_nocase_cmp);
 			break;
 		case HASH_DEFAULT:
 			list_init(&h->lists[i],
-					(int (*)(void*,void*))hash_item_cmp);
+				(int (*)(const void*,const void*))
+					hash_item_cmp);
 			break;
 		default:
 			fatal("wrong hash option %d", options);
@@ -458,6 +528,7 @@ void hash_clean(hash_t *h)
 	int i;
 	struct hash_item *hi;
 
+	assert(h);
 	for (i = 0; i < 256; i++) {
 		while ((hi = list_remove_first(&h->lists[i]))) {
 			free(hi->key);
@@ -468,6 +539,7 @@ void hash_clean(hash_t *h)
 
 void hash_free(hash_t *h)
 {
+	assert(h);
 	hash_clean(h);
 	free(h);
 }
@@ -475,15 +547,13 @@ void hash_free(hash_t *h)
 hash_t *hash_new(int options)
 {
 	hash_t *h;
-	h = malloc(sizeof(hash_t));
-	if (!h)
-		fatal("malloc");
+	h = bip_malloc(sizeof(hash_t));
 	hash_init(h, options);
 	return h;
 }
 
 /* Now we have a real hash, but we use only the last byte of it :p */
-static unsigned char hash_func(char *pkey)
+static unsigned char hash_func(const char *pkey)
 {
 	char c;
 	unsigned long hash = 5381; /* 5381 & 0xff makes more sense */
@@ -493,42 +563,58 @@ static unsigned char hash_func(char *pkey)
 	return (unsigned char)hash;
 }
 
-void hash_insert(hash_t *hash, char *key, void *ptr)
+void hash_insert(hash_t *hash, const char *key, void *ptr)
 {
 	struct hash_item *it;
 
+	assert(hash && key);
 	if (hash_get(hash, key))
 		fatal("Element with key %s already in hash %x\n", key, hash);
 
-	it = malloc(sizeof(struct hash_item));
-	if (!it)
-		fatal("malloc");
-	it->key = strdup(key);
+	it = bip_malloc(sizeof(struct hash_item));
+	it->key = bip_strdup(key);
 	it->item = ptr;
 	list_add_first(&hash->lists[hash_func(key)], it);
 }
 
-void *hash_get(hash_t *hash, char *key)
+int hash_includes(hash_t *hash, const char *key)
 {
 	struct hash_item *hi;
-	list_t *list = &hash->lists[hash_func(key)];
+	list_t *list;
+
+	assert(hash && key);
+	list = &hash->lists[hash_func(key)];
+	hi = list_get(list, key);
+	return hi != NULL;
+}
+
+void *hash_get(hash_t *hash, const char *key)
+{
+	struct hash_item *hi;
+	list_t *list;
+
+	assert(hash && key);
+	list = &hash->lists[hash_func(key)];
 	hi = list_get(list, key);
 	if (!hi)
 		return NULL;
 	return hi->item;
 }
 
-void *hash_remove_if_exists(hash_t *hash, char *key)
+void *hash_remove_if_exists(hash_t *hash, const char *key)
 {
+	assert(hash && key);
 	if (hash_get(hash, key) == NULL)
 		return NULL;
 	return hash_remove(hash, key);
 }
 
-void *hash_remove(hash_t *hash, char *key)
+void *hash_remove(hash_t *hash, const char *key)
 {
 	struct hash_item *it;
 	void *ptr;
+
+	assert(hash && key);
 	it = (struct hash_item *)list_remove(&hash->lists[hash_func(key)], key);
 	if (!it)
 		return NULL;
@@ -542,6 +628,7 @@ int hash_is_empty(hash_t *h)
 {
 	int i;
 
+	assert(h);
 	for (i = 0; i < 256; i++) {
 		if (!list_is_empty(&h->lists[i]))
 			return 0;
@@ -551,6 +638,7 @@ int hash_is_empty(hash_t *h)
 
 void hash_it_init(hash_t *h, hash_iterator_t *hi)
 {
+	assert(h);
 	memset(hi, 0, sizeof(hash_iterator_t));
 	hi->hash = h;
 
@@ -562,6 +650,7 @@ void hash_it_init(hash_t *h, hash_iterator_t *hi)
 
 void hash_it_next(hash_iterator_t *hi)
 {
+	assert(hi);
 	list_it_next(&hi->lit);
 	if (!list_it_item(&hi->lit)) {
 		do {
@@ -577,15 +666,18 @@ void *hash_it_item(hash_iterator_t *h)
 {
 	struct hash_item *hi;
 
+	assert(h);
 	hi = list_it_item(&h->lit);
 	if (!hi)
 		return NULL;
 	return hi->item;
 }
 
-char *hash_it_key(hash_iterator_t *h)
+const char *hash_it_key(hash_iterator_t *h)
 {
 	struct hash_item *hi;
+
+	assert(h);
 	hi = list_it_item(&h->lit);
 	if (!hi)
 		return NULL;
@@ -597,6 +689,7 @@ void *hash_it_remove(hash_iterator_t *hi)
 	struct hash_item *hitem;
 	void *ptr;
 
+	assert(hi);
 	hitem = list_it_remove(&hi->lit);
 
 	ptr = hitem->item;
@@ -608,15 +701,36 @@ void *hash_it_remove(hash_iterator_t *hi)
 void hash_dump(hash_t *h)
 {
 	hash_iterator_t it;
-	for (hash_it_init(h, &it); hash_it_item(&it) ;hash_it_next(&it))
+	assert(h);
+	for (hash_it_init(h, &it); hash_it_key(&it) ;hash_it_next(&it))
 		printf("%s => %p\n", hash_it_key(&it), hash_it_item(&it));
 }
 
-char *strmaydup(char *s)
+list_t *hash_keys(hash_t *hash)
+{
+	hash_iterator_t hi;
+	list_t *ret;
+
+	assert(hash);
+	ret = list_new(NULL);
+
+	for (hash_it_init(hash, &hi); hash_it_item(&hi); hash_it_next(&hi))
+		list_add_last(ret, bip_strdup(hash_it_key(&hi)));
+
+	return ret;
+}
+
+void hash_rename_key(hash_t *h, const char *oldk, const char *newk)
+{
+	assert(h && oldk && newk);
+	hash_insert(h, newk, hash_remove(h, oldk));
+}
+
+char *bip_strmaydup(char *s)
 {
 	if (!s)
 		return s;
-	return strdup(s);
+	return bip_strdup(s);
 }
 
 void strucase(char *s)
@@ -631,3 +745,80 @@ int ischannel(char p)
 {
 	return (p == '#' || p == '&' || p == '+' || p == '!');
 }
+
+void array_init(array_t *a)
+{
+	memset(a, 0, sizeof(array_t));
+}
+
+array_t *array_new(void)
+{
+	array_t *a;
+
+	a = bip_malloc(sizeof(array_t));
+	array_init(a);
+	return a;
+}
+
+void array_ensure(array_t *a, int index)
+{
+	assert(a && index >= 0);
+
+	if (array_includes(a, index))
+		return;
+	a->elemv = bip_realloc(a->elemv, sizeof(void *) * (index + 1));
+	memset(a->elemv + a->elemc, 0, sizeof(void *) * (index + 1 - a->elemc));
+	a->elemc = index + 1;
+}
+
+void *array_drop(array_t *a, int index)
+{
+	int i;
+	void *ret;
+
+	assert(a && array_includes(a, index));
+
+	ret = a->elemv[index];
+	for (i = index; i < array_count(a) - 1; i++)
+		a->elemv[i] = a->elemv[i + 1];
+	a->elemc--;
+	return ret;
+}
+
+array_t *array_extract(array_t *a, int index, int upto)
+{
+	array_t *ret;
+	int i;
+
+	assert(a && array_includes(a, index));
+	if (upto == -1)
+		upto = a->elemc;
+	assert((index == 0 && upto == 0) || array_includes(a, upto - 1));
+	assert(index <= upto);
+
+	ret = array_new();
+	if (index == upto)
+		return ret;
+
+	/* here we have index < upto */
+	array_ensure(ret, upto - index - 1);
+	for (i = 0; i < ret->elemc; i++)
+		ret->elemv[i] = a->elemv[i + index];
+	return ret;
+}
+
+void array_deinit(array_t *a)
+{
+	assert(a);
+	if (a->elemv)
+		free(a->elemv);
+	array_init(a);
+}
+
+void array_free(array_t *a)
+{
+	assert(a);
+	if (a->elemv)
+		free(a->elemv);
+	free(a);
+}
